@@ -923,6 +923,19 @@ class _Slice(CubePartition):
         # from the matrix later on.
         return self._columns_dimension._dimension_dict
 
+    @lazyproperty
+    def stddev(self):
+        """2D optional np.float64 ndarray of stddev value for each table cell.
+
+        Raises `ValueError` if the cube-result does not include a stddev cube-measure.
+        """
+        try:
+            return self._assembler.stddev
+        except ValueError:
+            raise ValueError(
+                "`.stddev` is undefined for a cube-result without a stddev measure"
+            )
+
     @lazyproperty
     def sums(self):
         """2D optional np.float64 ndarray of sum value for each table cell.
@@ -1422,6 +1435,20 @@ class _Strand(CubePartition):
         """dict, row dimension definition"""
         return self._rows_dimension._dimension_dict
 
+    @lazyproperty
+    def stddev(self):
+        """1D np.float64 ndarray of stddev for each row of strand.
+
+        Raises ValueError when accessed on a cube-result that does not contain a stddev
+        cube-measure.
+        """
+        try:
+            return self._assembler.stddev
+        except ValueError:
+            raise ValueError(
+                "`.stddev` is undefined for a cube-result without a stddev measure"
+            )
+
     @lazyproperty
     def sums(self):
         """1D np.float64 ndarray of sum for each row of strand.
