@@ -47,6 +47,11 @@ class StripeMeasures(object):
         """
         return _ScaledCounts(self._rows_dimension, self, self._cube_measures)
 
+    @lazyproperty
+    def stddev(self):
+        """_StdDev measure object for this stripe."""
+        return _StdDev(self._rows_dimension, self, self._cube_measures)
+
     @lazyproperty
     def sums(self):
         """_Sums measure object for this stripe."""
@@ -310,6 +315,27 @@ class _ScaledCounts(_BaseSecondOrderMeasure):
         return self._weighted_cube_counts.weighted_counts[self._has_numeric_value]
 
 
+class _StdDev(_BaseSecondOrderMeasure):
+    """Provides the stddev measure for a stripe.
+
+    Relies on the presence of a stddev cube-measure in the cube-result.
+    """
+
+    @lazyproperty
+    def base_values(self):
+        """1D np.float64 ndarray of stddev for each row."""
+        return self._cube_measures.cube_stddev.stddev
+
+    @lazyproperty
+    def subtotal_values(self):
+        """1D ndarray of np.nan for each row-subtotal.
+
+        StdDev values cannot be subtotaled and each subtotal value is unconditionally
+        np.nan.
+        """
+        return NanSubtotals.subtotal_values(self.base_values, self._rows_dimension)
+
+
 class _Sums(_BaseSecondOrderMeasure):
     """Provides the sum measure for a stripe.
 
@@ -323,12 +349,8 @@ class _Sums(_BaseSecondOrderMeasure):
 
     @lazyproperty
     def subtotal_values(self):
-        """1D ndarray of np.nan for each row-subtotal.
-
-        Sum values cannot be subtotaled and each subtotal value is unconditionally
-        np.nan.
-        """
-        return NanSubtotals.subtotal_values(self.base_values, self._rows_dimension)
+        """1D ndarray of sum subtotals for each row-subtotal."""
+        return SumSubtotals.subtotal_values(self.base_values, self._rows_dimension)
 
 
 class _TableProportionStddevs(_BaseSecondOrderMeasure):
