@@ -2,23 +2,32 @@
 
 """Unit test suite for `cr.cube.cube` module."""
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import pytest
 import numpy as np
 
-from cr.cube.cube import Cube, CubeSet, _Measures
+from cr.cube.cube import (
+    Cube,
+    CubeSet,
+    _Measures,
+    _ValidCountsMeasure,
+)
 from cr.cube.cubepart import _Slice, _Strand, _Nub
-from cr.cube.enum import DIMENSION_TYPE as DT
-from cr.cube.dimension import Dimension
+from cr.cube.enums import DIMENSION_TYPE as DT
 
 from ..fixtures import CR  # ---mnemonic: CR = 'cube-response'---
-from ..unitutil import call, class_mock, instance_mock, property_mock, method_mock
+from ..unitutil import call, class_mock, instance_mock, property_mock
 
 
 class DescribeCubeSet(object):
     """Unit-test suite for `cr.cube.cube.CubeSet` object."""
 
+    def it_knows_its_availabe_measures(self, cube_, _cubes_prop_):
+        cube_.available_measures = {"mean", "sum"}
+        _cubes_prop_.return_value = (cube_,)
+        cube_set = CubeSet(None, None, None, None)
+
+        assert cube_set.available_measures == {"mean", "sum"}
+
     @pytest.mark.parametrize(
         ("cubes_dimtypes", "expected_value"),
         (
@@ -59,27 +68,13 @@ class DescribeCubeSet(object):
 
         assert description == "Are you male or female?"
 
-    @pytest.mark.parametrize(
-        ("first_cube_has_means", "expected_value"), ((True, True), (False, False))
-    )
-    def it_knows_whether_it_has_means(
-        self, first_cube_has_means, expected_value, _cubes_prop_, cube_
-    ):
-        cube_.has_means = first_cube_has_means
-        _cubes_prop_.return_value = (cube_,)
-        cube_set = CubeSet(None, None, None, None)
-
-        has_means = cube_set.has_means
-
-        assert has_means == expected_value
-
     @pytest.mark.parametrize(
         ("first_cube_has_w_counts", "expected_value"), ((True, True), (False, False))
     )
     def it_knows_whether_it_has_weighted_counts(
         self, first_cube_has_w_counts, expected_value, _cubes_prop_, cube_
     ):
-        cube_.is_weighted = first_cube_has_w_counts
+        cube_.has_weighted_counts = first_cube_has_w_counts
         _cubes_prop_.return_value = (cube_,)
         cube_set = CubeSet(None, None, None, None)
 
@@ -159,43 +154,12 @@ class DescribeCubeSet(object):
 
         np.testing.assert_almost_equal(cubeset_population_fraction, expected_value)
 
-    def it_provides_access_to_its_Cube_objects_to_help(self, _iter_cubes_, cube_):
-        _iter_cubes_.return_value = (c for c in (cube_, cube_, cube_))
-        cube_set = CubeSet(None, None, None, None)
-
-        cubes = cube_set._cubes
-
-        assert cubes == (cube_, cube_, cube_)
-
-    @pytest.mark.parametrize(
-        ("is_multi_cube", "cube_0_ndim", "expected_value"),
-        ((False, 1, False), (False, 0, False), (True, 1, False), (True, 0, True)),
-    )
-    def it_knows_whether_it_is_numeric_mean_to_help(
-        self,
-        _is_multi_cube_prop_,
-        is_multi_cube,
-        Cube_,
-        cube_,
-        cube_0_ndim,
-        expected_value,
-    ):
-        _is_multi_cube_prop_.return_value = is_multi_cube
-        cube_.ndim = cube_0_ndim
-        Cube_.return_value = cube_
-        cube_set = CubeSet(({"cube": 0}, {"cube": 1}), None, None, None)
-
-        is_numeric_mean = cube_set._is_numeric_mean
-
-        assert Cube_.call_args_list == ([call({"cube": 0})] if is_multi_cube else [])
-        assert is_numeric_mean == expected_value
-
     def it_constructs_its_sequence_of_cube_objects_to_help(
-        self, request, Cube_, _is_numeric_mean_prop_
+        self, request, Cube_, _is_numeric_measure_prop_
     ):
         cubes_ = tuple(instance_mock(request, Cube) for _ in range(4))
         Cube_.side_effect = iter(cubes_)
-        _is_numeric_mean_prop_.return_value = False
+        _is_numeric_measure_prop_.return_value = False
         cube_set = CubeSet(
             cube_responses=[{"cube": "resp-1"}, {"cube": "resp-2"}, {"cube": "resp-3"}],
             transforms=[{"xfrms": 1}, {"xfrms": 2}, {"xfrms": 3}],
@@ -203,7 +167,7 @@ class DescribeCubeSet(object):
             min_base=10,
         )
 
-        cubes = tuple(cube_set._iter_cubes())
+        cubes = cube_set._cubes
 
         assert Cube_.call_args_list == [
             call(
@@ -231,12 +195,12 @@ class DescribeCubeSet(object):
         assert cubes == cubes_[:3]
 
     def but_it_inflates_the_cubes_in_special_case_of_numeric_mean_payload(
-        self, request, Cube_, cube_, _is_numeric_mean_prop_
+        self, request, Cube_, cube_, _is_numeric_measure_prop_
     ):
         cubes_ = tuple(instance_mock(request, Cube) for _ in range(4))
         cube_.inflate.side_effect = iter(cubes_)
         Cube_.return_value = cube_
-        _is_numeric_mean_prop_.return_value = True
+        _is_numeric_measure_prop_.return_value = True
         cube_set = CubeSet(
             cube_responses=[{"cube": "resp-1"}, {"cube": "resp-2"}, {"cube": "resp-3"}],
             transforms=[{"xfrms": 1}, {"xfrms": 2}, {"xfrms": 3}],
@@ -244,7 +208,7 @@ class DescribeCubeSet(object):
             min_base=10,
         )
 
-        cubes = tuple(cube_set._iter_cubes())
+        cubes = cube_set._cubes
 
         assert Cube_.call_args_list == [
             call(
@@ -272,6 +236,47 @@ class DescribeCubeSet(object):
         assert cube_.inflate.call_args_list == [call(), call(), call()]
         assert cubes == cubes_[:3]
 
+    @pytest.mark.parametrize(
+        ("is_multi_cube", "cube_0_ndim", "expected_value"),
+        ((False, 1, False), (False, 0, False), (True, 1, False), (True, 0, True)),
+    )
+    def it_knows_whether_it_is_numeric_measure_to_help(
+        self,
+        _is_multi_cube_prop_,
+        is_multi_cube,
+        Cube_,
+        cube_,
+        cube_0_ndim,
+        expected_value,
+    ):
+        _is_multi_cube_prop_.return_value = is_multi_cube
+        cube_.ndim = cube_0_ndim
+        Cube_.return_value = cube_
+        cube_set = CubeSet(({"cube": 0}, {"cube": 1}), None, None, None)
+
+        is_numeric_mean = cube_set._is_numeric_measure
+
+        assert Cube_.call_args_list == ([call({"cube": 0})] if is_multi_cube else [])
+        assert is_numeric_mean == expected_value
+
+    def it_knows_its_valid_counts_summary_to_help(self, _cubes_prop_, cube_):
+        cube_.valid_counts_summary = np.array([1, 2, 3])
+        _cubes_prop_.return_value = (cube_,)
+        cube_set = CubeSet(None, None, None, None)
+
+        valid_counts_summary = cube_set.valid_counts_summary
+
+        np.testing.assert_array_equal(valid_counts_summary, [1, 2, 3])
+
+    def it_knows_its_n_reposnes_to_help(self, _cubes_prop_, cube_):
+        cube_.n_responses = 6
+        _cubes_prop_.return_value = (cube_,)
+        cube_set = CubeSet(None, None, None, None)
+
+        n_responses = cube_set.n_responses
+
+        assert n_responses == 6
+
     # fixture components ---------------------------------------------
 
     @pytest.fixture
@@ -291,12 +296,8 @@ class DescribeCubeSet(object):
         return property_mock(request, CubeSet, "_is_multi_cube")
 
     @pytest.fixture
-    def _is_numeric_mean_prop_(self, request):
-        return property_mock(request, CubeSet, "_is_numeric_mean")
-
-    @pytest.fixture
-    def _iter_cubes_(self, request):
-        return method_mock(request, CubeSet, "_iter_cubes")
+    def _is_numeric_measure_prop_(self, request):
+        return property_mock(request, CubeSet, "_is_numeric_measure")
 
 
 class DescribeCube(object):
@@ -314,7 +315,18 @@ class DescribeCube(object):
 
     def it_can_inflate_itself(self, request):
         cube = Cube(
-            {"result": {"dimensions": [{"other": "dim"}]}},
+            {
+                "result": {
+                    "dimensions": [{"other": "dim"}],
+                    "measures": {
+                        "mean": {
+                            "metadata": {
+                                "references": {"alias": "mean", "name": "Mean"}
+                            }
+                        }
+                    },
+                }
+            },
             cube_idx=1,
             transforms={"trans": "forms"},
             population=1000,
@@ -332,15 +344,22 @@ class DescribeCube(object):
                 "result": {
                     "dimensions": [
                         {
-                            "references": {"alias": "mean", "name": "mean"},
+                            "references": {"alias": "mean", "name": "Mean"},
                             "type": {
-                                "class": "categorical",
                                 "categories": [{"id": 1, "name": "Mean"}],
+                                "class": "categorical",
                             },
                         },
                         {"other": "dim"},
-                    ]
-                }
+                    ],
+                    "measures": {
+                        "mean": {
+                            "metadata": {
+                                "references": {"alias": "mean", "name": "Mean"}
+                            }
+                        }
+                    },
+                },
             },
             1,
             {"trans": "forms"},
@@ -355,37 +374,6 @@ class DescribeCube(object):
     def it_knows_its_index_within_its_cube_set(self, cube_idx_arg, expected_value):
         assert Cube(None, cube_idx_arg).cube_index == expected_value
 
-    @pytest.mark.parametrize(
-        ("dim_types", "aliases", "expected_value"),
-        (
-            ((), (), False),
-            ((DT.MR,), ("alias",), False),
-            ((DT.MR, DT.MR), ("alias", "alias"), False),
-            ((DT.MR, DT.CAT, DT.CAT), ("alias1", "alias2", "alias2"), False),
-            ((DT.CAT, DT.MR, DT.MR), ("alias1", "alias2", "alias3"), False),
-            ((DT.CAT, DT.MR, DT.MR), ("alias1", "alias2", "alias2"), True),
-        ),
-    )
-    def it_knows_if_it_is_mr_aug(
-        self, request, dim_types, aliases, expected_value, dimension_types_prop_
-    ):
-        property_mock(request, Cube, "ndim", return_value=len(dim_types))
-        dimension_types_prop_.return_value = dim_types
-        property_mock(
-            request,
-            Cube,
-            "dimensions",
-            return_value=[
-                instance_mock(request, Dimension, alias=aliases[i])
-                for i, _ in enumerate(dim_types)
-            ],
-        )
-        cube = Cube(None, None, None, None)
-
-        is_mr_aug = cube.is_mr_aug
-
-        assert is_mr_aug is expected_value
-
     @pytest.mark.parametrize(
         ("dim_types", "cube_idx", "_is_single_filter_col_cube", "expected_value"),
         (
@@ -453,7 +441,7 @@ class DescribeCube(object):
         ((CR.CAT_X_CAT, CR.CAT_X_CAT), ({"value": "val"}, "val")),
     )
     def and_it_accepts_a_JSON_format_cube_response(self, cube_response, expected_value):
-        assert Cube(cube_response)._cube_dict == expected_value
+        assert Cube(cube_response)._cube_response == expected_value
 
     @pytest.mark.parametrize(
         ("cube_response", "expected_value"),
@@ -470,18 +458,208 @@ class DescribeCube(object):
             ),
         ),
     )
-    def but_it_raises_on_other_cube_response_types(self, cube_response, expected_value):
+    def but_it_raises_on_other_cube_response_types(
+        self,
+        cube_response,
+        expected_value,
+    ):
         with pytest.raises(TypeError) as e:
             Cube(cube_response)._cube_dict
 
         assert str(e.value) == expected_value
 
+    @pytest.mark.parametrize(
+        "numeric_subvariables, num_measure_references, expected_value",
+        (
+            (
+                ["001", "002"],
+                {
+                    "subreferences": [
+                        {"alias": "A", "name": "A"},
+                        {"alias": "B", "name": "B"},
+                    ]
+                },
+                [
+                    {
+                        "id": 0,
+                        "value": {
+                            "id": "001",
+                            "references": {"alias": "A", "name": "A"},
+                        },
+                    },
+                    {
+                        "id": 1,
+                        "value": {
+                            "id": "002",
+                            "references": {"alias": "B", "name": "B"},
+                        },
+                    },
+                ],
+            ),
+        ),
+    )
+    def it_knows_its_num_array_dimensions(
+        self,
+        _numeric_references_prop_,
+        _numeric_subvariables_prop_,
+        numeric_subvariables,
+        num_measure_references,
+        expected_value,
+    ):
+        _numeric_references_prop_.return_value = num_measure_references
+        _numeric_subvariables_prop_.return_value = numeric_subvariables
+        cube = Cube(None)
+
+        _num_array_dimensions = cube._numeric_array_dimension
+
+        assert _num_array_dimensions["type"]["elements"] == expected_value
+
+    def but_it_returns_None_when_numeric_subvars_is_empty(
+        self, _numeric_subvariables_prop_
+    ):
+        _numeric_subvariables_prop_.return_value = []
+        cube = Cube(None)
+
+        _num_array_dimensions = cube._numeric_array_dimension
+
+        assert _num_array_dimensions is None
+
+    @pytest.mark.parametrize(
+        "cube_response, expected_value",
+        (
+            ({}, []),
+            ({"foo": "bar"}, []),
+            (
+                {
+                    "result": {
+                        "measures": {
+                            "mean": {"metadata": {"type": {"subvariables": ["A", "B"]}}}
+                        }
+                    }
+                },
+                ["A", "B"],
+            ),
+        ),
+    )
+    def it_knows_its_numeric_subvariables(
+        self, _cube_response_prop_, cube_response, expected_value
+    ):
+        _cube_response_prop_.return_value = cube_response
+        cube = Cube(None)
+
+        numeric_measure_subvariables = cube._numeric_measure_subvariables
+
+        assert numeric_measure_subvariables == expected_value
+
+    @pytest.mark.parametrize(
+        "cube_response, expected_value",
+        (
+            ({}, {}),
+            ({"foo": "bar"}, {}),
+            (
+                {
+                    "result": {
+                        "measures": {
+                            "mean": {
+                                "metadata": {
+                                    "references": {
+                                        "subreferences": [
+                                            {"name": "A", "alias": "A"},
+                                            {"name": "B", "alias": "B"},
+                                        ]
+                                    }
+                                }
+                            }
+                        }
+                    }
+                },
+                {
+                    "subreferences": [
+                        {"name": "A", "alias": "A"},
+                        {"name": "B", "alias": "B"},
+                    ]
+                },
+            ),
+        ),
+    )
+    def it_knows_its_numeric_references(
+        self, _cube_response_prop_, cube_response, expected_value
+    ):
+        _cube_response_prop_.return_value = cube_response
+        cube = Cube(None)
+
+        numeric_references = cube._numeric_measure_references
+
+        assert numeric_references == expected_value
+
+    @pytest.mark.parametrize(
+        "cube_response, cube_idx_arg, numeric_subvars, num_array_dim, expected_value",
+        (
+            ({}, None, [], {}, {}),
+            ({"result": {"foo": "bar"}}, None, [], {}, {"result": {"foo": "bar"}}),
+            (
+                {"result": {"foo": "bar"}},
+                None,
+                ["A", "B"],
+                {},
+                {"result": {"foo": "bar"}},
+            ),
+            (
+                {"result": {"dimensions": []}},
+                None,
+                ["A", "B"],
+                {"A": "B"},
+                {"result": {"dimensions": [{"A": "B"}]}},
+            ),
+            (
+                {"result": {"dimensions": ["A", "B"]}},
+                1,
+                ["A", "B"],
+                {"A": "B"},
+                {"result": {"dimensions": [{"A": "B"}, "A", "B"]}},
+            ),
+        ),
+    )
+    def it_knows_its_cube_dict(
+        self,
+        cube_response,
+        cube_idx_arg,
+        numeric_subvars,
+        num_array_dim,
+        expected_value,
+        _cube_response_prop_,
+        _numeric_subvariables_prop_,
+        _numeric_array_dimension_prop_,
+    ):
+        _cube_response_prop_.return_value = cube_response
+        _numeric_subvariables_prop_.return_value = numeric_subvars
+        _numeric_array_dimension_prop_.return_value = num_array_dim
+        cube = Cube(None, cube_idx=cube_idx_arg)
+
+        assert cube._cube_dict == expected_value
+
     # fixture components ---------------------------------------------
 
     @pytest.fixture
     def _cube_dict_prop_(self, request):
         return property_mock(request, Cube, "_cube_dict")
 
+    @pytest.fixture
+    def _cube_response_prop_(self, request):
+        return property_mock(request, Cube, "_cube_response")
+
+    @pytest.fixture
+    def _numeric_references_prop_(self, request):
+        return property_mock(request, Cube, "_numeric_measure_references")
+
+    @pytest.fixture
+    def _numeric_subvariables_prop_(self, request):
+        return property_mock(request, Cube, "_numeric_measure_subvariables")
+
+    @pytest.fixture
+    def _numeric_array_dimension_prop_(self, request):
+        return property_mock(request, Cube, "_numeric_array_dimension")
+
     @pytest.fixture
     def dimension_types_prop_(self, request):
         return property_mock(request, Cube, "dimension_types")
@@ -519,3 +697,15 @@ class DescribeMeasures(object):
         population_fraction = measures.population_fraction
 
         np.testing.assert_equal(population_fraction, expected_value)
+
+
+class Describe_ValidCountsMeasure(object):
+    @pytest.mark.parametrize(
+        "valid_counts, expected_value",
+        (({"valid_count_unweighted": {"data": [3, 2, 1]}}, [3, 2, 1]), ({}, None)),
+    )
+    def it_knows_its_flat_values(self, valid_counts, expected_value):
+        cube_dict = {"result": {"measures": valid_counts}}
+        valid_counts = _ValidCountsMeasure(cube_dict, None)
+
+        np.testing.assert_equal(valid_counts._flat_values, expected_value)
