@@ -2,8 +2,6 @@
 
 """Unit test suite for cr.cube.dimension module."""
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import numpy as np
 import pytest
 
@@ -22,7 +20,7 @@ from cr.cube.dimension import (
     _Subtotals,
     _ValidElements,
 )
-from cr.cube.enum import DIMENSION_TYPE as DT
+from cr.cube.enums import COLLATION_METHOD as CM, DIMENSION_TYPE as DT
 
 from ..unitutil import (
     ANY,
@@ -36,7 +34,7 @@ from ..unitutil import (
 
 
 class Describe_BaseDimensions(object):
-    def it_has_sequence_behaviors(self, request, _dimensions_prop_):
+    def it_has_sequence_behaviors(self, _dimensions_prop_):
         _dimensions_prop_.return_value = (0, 1, 2)
         base_dimensions = _BaseDimensions()
 
@@ -448,6 +446,28 @@ class Describe_RawDimension(object):
 class DescribeDimension(object):
     """Unit-test suite for `cr.cube.dimension.Dimension` object."""
 
+    @pytest.mark.parametrize(
+        "dimension_dict, expected_value",
+        (
+            ({"references": {}}, None),
+            ({"references": {"alias": "Elias"}}, "Elias"),
+        ),
+    )
+    def it_knows_its_alias(self, dimension_dict, expected_value):
+        assert Dimension(dimension_dict, None).alias == expected_value
+
+    @pytest.mark.parametrize(
+        "order_dict, expected_value",
+        (
+            ({}, CM.PAYLOAD_ORDER),
+            ({"type": "explicit"}, CM.EXPLICIT_ORDER),
+            ({"type": "payload_order"}, CM.PAYLOAD_ORDER),
+        ),
+    )
+    def it_knows_its_collation_method(self, request, order_dict, expected_value):
+        property_mock(request, Dimension, "order_dict", return_value=order_dict)
+        assert Dimension(None, None).collation_method == expected_value
+
     @pytest.mark.parametrize(
         "dimension_dict, expected_value",
         (
@@ -458,16 +478,22 @@ class DescribeDimension(object):
         ),
     )
     def it_knows_its_description(self, dimension_dict, expected_value):
-        dimension = Dimension(dimension_dict, None)
-
-        description = dimension.description
-
-        assert description == expected_value
+        assert Dimension(dimension_dict, None).description == expected_value
 
     def it_knows_its_dimension_type(self):
-        dimension = Dimension(None, DT.CAT)
-        dimension_type = dimension.dimension_type
-        assert dimension_type == DT.CAT
+        assert Dimension(None, DT.CAT).dimension_type == DT.CAT
+
+    def it_knows_its_element_ids(self, request, valid_elements_prop_):
+        valid_elements_prop_.return_value = (
+            instance_mock(request, _Element, element_id=(i + 1)) for i in range(6)
+        )
+        assert Dimension(None, None).element_ids == (1, 2, 3, 4, 5, 6)
+
+    def it_computes_its_hidden_idxs(self, request, valid_elements_prop_):
+        valid_elements_prop_.return_value = (
+            instance_mock(request, _Element, is_hidden=bool(i % 2)) for i in range(7)
+        )
+        assert Dimension(None, None).hidden_idxs == (1, 3, 5)
 
     def it_knows_the_numeric_values_of_its_elements(
         self, request, valid_elements_prop_
@@ -476,11 +502,37 @@ class DescribeDimension(object):
             instance_mock(request, _Element, numeric_value=numeric_value)
             for numeric_value in (1, 2.2, np.nan)
         )
-        dimension = Dimension(None, None)
+        assert Dimension(None, None).numeric_values == (1, 2.2, np.nan)
 
-        numeric_values = dimension.numeric_values
+    @pytest.mark.parametrize(
+        "dimension_transforms, expected_value",
+        (
+            (None, {}),
+            ({}, {}),
+            ({"foo": "bar"}, {}),
+            ({"order": {"order": "dict"}}, {"order": "dict"}),
+        ),
+    )
+    def it_provides_access_to_the_transforms_order_dict(
+        self, dimension_transforms, expected_value
+    ):
+        assert Dimension(None, None, dimension_transforms).order_dict == expected_value
 
-        assert numeric_values == (1, 2.2, np.nan)
+    @pytest.mark.parametrize(
+        "selected_categories, expected_value",
+        (
+            ({"selected_categories": None}, ()),
+            ({"foo": "bar"}, ()),
+            ({}, ()),
+            (
+                {"selected_categories": [{"name": "Very Fav.", "id": 1}]},
+                ({"name": "Very Fav.", "id": 1},),
+            ),
+        ),
+    )
+    def it_knows_its_selected_categories(self, selected_categories, expected_value):
+        dimension_dict = {"references": selected_categories}
+        assert Dimension(dimension_dict, None).selected_categories == expected_value
 
     @pytest.mark.parametrize(
         "dimension_dict, insertion_dicts",
@@ -515,46 +567,57 @@ class DescribeDimension(object):
         subtotals_,
         valid_elements_prop_,
         valid_elements_,
-        prune_prop_,
     ):
         dimension_dict["type"] = {"class": "categorical", "categories": []}
         valid_elements_prop_.return_value = valid_elements_
         _Subtotals_.return_value = subtotals_
-        prune_prop_.return_value = True
         dimension = Dimension(dimension_dict, None)
 
         subtotals = dimension.subtotals
 
-        _Subtotals_.assert_called_once_with(insertion_dicts, valid_elements_, True)
+        _Subtotals_.assert_called_once_with(insertion_dicts, valid_elements_)
         assert subtotals is subtotals_
 
-    def but_it_overrides_with_subtotal_transforms_when_present(
+    @pytest.mark.parametrize("dimension_type", (DT.MR, DT.CA_SUBVAR))
+    def but_it_suppresses_any_subtotals_on_an_array_dimension(
+        self,
+        request,
+        dimension_type,
+        _Subtotals_,
+        subtotals_,
+        valid_elements_prop_,
+        valid_elements_,
+    ):
+        property_mock(request, Dimension, "dimension_type", return_value=dimension_type)
+        _Subtotals_.return_value = subtotals_
+        valid_elements_prop_.return_value = valid_elements_
+        dimension_dict = {"references": {"view": {"transform": {"insertions": [666]}}}}
+        dimension = Dimension(dimension_dict, None)
+
+        subtotals = dimension.subtotals
+
+        _Subtotals_.assert_called_once_with([], valid_elements_)
+        assert subtotals is subtotals_
+
+    def and_it_overrides_with_subtotal_transforms_when_present(
         self,
         valid_elements_prop_,
         valid_elements_,
         _Subtotals_,
         subtotals_,
-        prune_prop_,
     ):
         dimension_transforms = {"insertions": ["subtotal", "dicts"]}
         valid_elements_prop_.return_value = valid_elements_
         _Subtotals_.return_value = subtotals_
-        prune_prop_.return_value = False
         dimension = Dimension(None, None, dimension_transforms)
 
         subtotals = dimension.subtotals
 
-        _Subtotals_.assert_called_once_with(
-            ["subtotal", "dicts"], valid_elements_, False
-        )
+        _Subtotals_.assert_called_once_with(["subtotal", "dicts"], valid_elements_)
         assert subtotals is subtotals_
 
     # fixture components ---------------------------------------------
 
-    @pytest.fixture
-    def prune_prop_(self, request):
-        return property_mock(request, Dimension, "prune")
-
     @pytest.fixture
     def _Subtotals_(self, request):
         return class_mock(request, "cr.cube.dimension._Subtotals")
@@ -669,12 +732,10 @@ class Describe_AllElements(object):
     def it_creates_its_Element_objects_in_a_local_factory_to_help(
         self,
         request,
-        _prune_prop_,
         _ElementTransforms_,
         _Element_,
         _iter_element_makings_,
     ):
-        _prune_prop_.return_value = True
         element_transforms_ = tuple(
             instance_mock(request, _ElementTransforms, name="element-xfrms-%s" % idx)
             for idx in range(3)
@@ -697,9 +758,9 @@ class Describe_AllElements(object):
         elements = all_elements._elements
 
         assert _ElementTransforms_.call_args_list == [
-            call({"xfrms": 0}, True),
-            call({"xfrms": 1}, True),
-            call({"xfrms": 2}, True),
+            call({"xfrms": 0}),
+            call({"xfrms": 1}),
+            call({"xfrms": 2}),
         ]
         assert _Element_.call_args_list == [
             call({"element": "dict-A"}, 0, element_transforms_[0]),
@@ -749,10 +810,6 @@ class Describe_AllElements(object):
     def _iter_element_makings_(self, request):
         return method_mock(request, _AllElements, "_iter_element_makings")
 
-    @pytest.fixture
-    def _prune_prop_(self, request):
-        return property_mock(request, _AllElements, "_prune")
-
     @pytest.fixture
     def _ValidElements_(self, request):
         return class_mock(request, "cr.cube.dimension._ValidElements")
@@ -949,7 +1006,7 @@ class DescribeElementTransforms(object):
         ),
     )
     def it_knows_its_fill_color_value(self, element_transforms_dict, expected_value):
-        element_transforms = _ElementTransforms(element_transforms_dict, None)
+        element_transforms = _ElementTransforms(element_transforms_dict)
 
         fill_color_value = element_transforms.fill
 
@@ -967,7 +1024,7 @@ class DescribeElementTransforms(object):
     def it_knows_when_it_is_explicitly_hidden(
         self, element_transforms_dict, expected_value
     ):
-        element_transforms = _ElementTransforms(element_transforms_dict, None)
+        element_transforms = _ElementTransforms(element_transforms_dict)
 
         is_hidden = element_transforms.hide
 
@@ -983,7 +1040,7 @@ class DescribeElementTransforms(object):
         ),
     )
     def it_knows_its_name(self, element_transforms_dict, expected_value):
-        element_transforms = _ElementTransforms(element_transforms_dict, None)
+        element_transforms = _ElementTransforms(element_transforms_dict)
 
         name = element_transforms.name
 
@@ -995,28 +1052,16 @@ class Describe_Subtotals(object):
 
     def it_has_sequence_behaviors(self, request, _subtotals_prop_):
         _subtotals_prop_.return_value = (1, 2, 3)
-        subtotals = _Subtotals(None, None, None)
+        subtotals = _Subtotals(None, None)
 
         assert subtotals[1] == 2
         assert subtotals[1:3] == (2, 3)
         assert len(subtotals) == 3
         assert list(n for n in subtotals) == [1, 2, 3]
 
-    def it_can_iterate_subtotals_having_a_given_anchor(self, request, _subtotals_prop_):
-        subtotals_ = tuple(
-            instance_mock(request, _Subtotal, name="subtotal-%d" % idx, anchor=anchor)
-            for idx, anchor in enumerate(["bottom", 2, "bottom"])
-        )
-        _subtotals_prop_.return_value = subtotals_
-        subtotals = _Subtotals(None, None, None)
-
-        subtotals_with_anchor = tuple(subtotals.iter_for_anchor("bottom"))
-
-        assert subtotals_with_anchor == (subtotals_[0], subtotals[2])
-
     def it_provides_the_element_ids_as_a_set_to_help(self, request, valid_elements_):
         valid_elements_.element_ids = tuple(range(3))
-        subtotals = _Subtotals(None, valid_elements_, None)
+        subtotals = _Subtotals(None, valid_elements_)
 
         element_ids = subtotals._element_ids
 
@@ -1058,7 +1103,7 @@ class Describe_Subtotals(object):
         self, insertion_dicts, element_ids, expected_value, _element_ids_prop_
     ):
         _element_ids_prop_.return_value = element_ids
-        subtotals = _Subtotals(insertion_dicts, None, None)
+        subtotals = _Subtotals(insertion_dicts, None)
 
         subtotal_dicts = tuple(subtotals._iter_valid_subtotal_dicts())
 
@@ -1074,13 +1119,14 @@ class Describe_Subtotals(object):
         )
         _iter_valid_subtotal_dicts_.return_value = iter(subtotal_dicts_)
         _Subtotal_.side_effect = iter(subtotal_objs_)
-        subtotals = _Subtotals(None, valid_elements_, True)
+        subtotals = _Subtotals(None, valid_elements_)
 
         subtotal_objs = subtotals._subtotals
 
         assert _Subtotal_.call_args_list == [
-            call(subtot_dict_, valid_elements_, True)
-            for subtot_dict_ in subtotal_dicts_
+            call(subtotal_dicts_[0], valid_elements_, 1),
+            call(subtotal_dicts_[1], valid_elements_, 2),
+            call(subtotal_dicts_[2], valid_elements_, 3),
         ]
         assert subtotal_objs == subtotal_objs_
 
@@ -1161,9 +1207,7 @@ class Describe_Subtotal(object):
         valid_elements_.element_ids = element_ids
         subtotal = _Subtotal(subtotal_dict, valid_elements_, None)
 
-        anchor = subtotal.anchor
-
-        assert anchor == expected_value
+        assert subtotal.anchor == expected_value
 
     @pytest.mark.parametrize(
         "subtotal_dict, element_ids, expected_value",
@@ -1183,20 +1227,75 @@ class Describe_Subtotal(object):
         valid_elements_.element_ids = element_ids
         subtotal = _Subtotal(subtotal_dict, valid_elements_, None)
 
-        addend_ids = subtotal.addend_ids
+        assert subtotal.addend_ids == expected_value
 
-        assert addend_ids == expected_value
+    def it_knows_its_insertion_id_when_it_has_been_assigned_one(self):
+        assert _Subtotal({"insertion_id": 42}, None, None).insertion_id == 42
+
+    def and_it_uses_its_fallback_insertion_id_when_not_assigned_one(self):
+        assert _Subtotal({}, None, 24).insertion_id == 24
 
     @pytest.mark.parametrize(
         "subtotal_dict, expected_value",
         (({}, ""), ({"name": None}, ""), ({"name": ""}, ""), ({"name": "Joe"}, "Joe")),
     )
     def it_knows_the_subtotal_label(self, subtotal_dict, expected_value):
-        subtotal = _Subtotal(subtotal_dict, None, None)
+        assert _Subtotal(subtotal_dict, None, None).label == expected_value
 
-        label = subtotal.label
+    @pytest.mark.parametrize(
+        "subtotal_dict, element_ids, expected_value",
+        (
+            ({}, {}, ()),
+            ({"kwargs": {"negative": [1]}}, {1}, (1,)),
+            ({"kwargs": {"negative": [1, 2, 3]}}, {1, 2, 3}, (1, 2, 3)),
+            ({"kwargs": {"negative": [1, 2, 3]}}, {1, 3}, (1, 3)),
+            ({"kwargs": {"negative": [3, 2]}}, {1, 2, 3}, (3, 2)),
+            ({"kwargs": {"negative": []}}, {1, 2, 3}, ()),
+            ({"kwargs": {"negative": [1, 2, 3]}}, {}, ()),
+        ),
+    )
+    def it_provides_access_to_the_subtrahend_element_ids(
+        self, subtotal_dict, element_ids, expected_value, valid_elements_
+    ):
+        valid_elements_.element_ids = element_ids
+        subtotal = _Subtotal(subtotal_dict, valid_elements_, None)
 
-        assert label == expected_value
+        assert subtotal.subtrahend_ids == expected_value
+
+    @pytest.mark.parametrize(
+        "subtotal_dict, subtrahend_ids, expected_value",
+        (
+            ({"kwargs": {"negative": []}}, (), []),
+            ({"kwargs": {"negative": [1, 2]}}, (1, 2), [0, 1]),
+            ({"kwargs": {"negative": [0, 2]}}, (1, 2), [0, 1]),
+            ({"kwargs": {"negative": [3, 4]}}, (3, 4), [2, 3]),
+            ({"kwargs": {"negative": [4, 99]}}, (4, 99), [3, 4]),
+        ),
+    )
+    def it_knows_the_subtrahend_idxs(
+        self,
+        subtrahend_ids_,
+        all_elements_,
+        valid_elements_,
+        subtotal_dict,
+        subtrahend_ids,
+        expected_value,
+    ):
+        subtrahend_ids_.return_value = subtrahend_ids
+        elements_ = (
+            _Element({"id": 1}, None, None),
+            _Element({"id": 2}, None, None),
+            _Element({"id": 3}, None, None),
+            _Element({"id": 4}, None, None),
+            _Element({"id": 99}, None, None),
+        )
+        all_elements_.__iter__.return_value = iter(elements_)
+        valid_elements = _ValidElements(all_elements_, None)
+        subtotal = _Subtotal(subtotal_dict, valid_elements, None)
+
+        subtrahend_idxs = subtotal.subtrahend_idxs
+
+        np.testing.assert_array_almost_equal(subtrahend_idxs, expected_value)
 
     # fixture components ---------------------------------------------
 
@@ -1208,6 +1307,10 @@ class Describe_Subtotal(object):
     def all_elements_(self, request):
         return instance_mock(request, _AllElements)
 
+    @pytest.fixture
+    def subtrahend_ids_(self, request):
+        return property_mock(request, _Subtotal, "subtrahend_ids")
+
     @pytest.fixture
     def valid_elements_(self, request):
         return instance_mock(request, _ValidElements)
